# QGIS Web Client 2 Documentation

## Introduction

QGIS Web Client 2 (QWC2) is a modular next generation responsive web client for QGIS Server, built with ReactJS and OpenLayers.

It is released under the terms of the [BSD license](https://github.com/qgis/qwc2-demo-app/blob/master/LICENSE).

## Requirements

An working QGIS server environment is required. Projects which are to be offered as QWC2 topics must be published as WMS.

Furthermore, a web server is required which will serve the QWC2 application.

If possible, the QGIS server should run on the same scheme, hostname and port as the QWC2 application, otherwise additional measures need to be taken to allow cross-origin QWC2 requests see [Cross-Origin requests](#cross-origin-requests).

## <a name="quick-start"></a>Quick start

QWC2 is divided into two repositories:

- The QWC2 components, hosted at [https://github.com/qgis/qwc2/](https://github.com/qgis/qwc2/). This repository contains the core building blocks common to all QWC2 applications.
- The QWC2 application, an example is hosted at [https://github.com/qgis/qwc2-demo-app](https://github.com/qgis/qwc2-demo-app). This repository contains the user-specific configuration of the core components and can also include any other custom components.

Additionally, some QWC2 components (such as permalink generation, elevation queries, etc) require external services. Example implementations of such services are hosted at [https://github.com/sourcepole/qwc2-server](https://github.com/sourcepole/qwc2-server).

To work with QWC2, you will need a minimal development environment consisting of [git](https://git-scm.com/), [node](https://nodejs.org/) and [yarn](https://yarnpkg.com).

The fastest way to get started is by recursively cloning the demo application repository:

    $ git clone --recursive https://github.com/qgis/qwc2-demo-app.git

Next, install all required dependencies:

    $ cd qwc2-demo-app
    $ yarn install

Then, start a local development application:

    $ yarn start

The development application will run by default on `http://localhost:8081`.

At this point, you can customize and configure the application according to your needs, as described in detail in the following chapters.

The final step is to compile a deployable application bundle for production:

    $ yarn run prod

You can then deploy the contents of the `prod` folder to your web server root. If you opt to deploy the application to a subfolder of the web server root, you will need to adjust the `assetsPath` and` translationsPath` settings in `config.json` accordingly.


## QWC2 configuration

The layout of the development tree is as follows:

| Path                      | Description
|---------------------------|-------------------------------------------------------------------|
|`├─ assets/`               |                                                                   |
|`│  ├─ css/qwc2.css`       | Master style sheet                                                |
|`│  ├─ img/`               | Application logo                                                  |
|`│  │  └─ mapthumbs/`      | Map thumbnails, typically autogenerated                           |
|`│  └─ templates/`         |                                                                   |
|`│     └─ legendprint.html`| HTML template for the legend print                                |
|`├─ js/`                   |                                                                   |
|`│  ├─ app.jsx`            | Entry point of the ReactJS application                            |
|`│  ├─ appConfig.js`       | Configuration of the qwc2 core modules                            |
|`│  ├─ EditingInterface.js`| Interface to the service for editing features, see [EditingInterface.js](#editing-interface). |
|`│  ├─ Help.jsx`           | Component for rendering a user-defined Help dialog                |
|`│  └─ SearchProviders.js` | Search providers                                                  |
|`├─ icons/`                | Application icons                                                |
|`├─ qwc2/`                 | Git submodule containing the core qwc2 components                 |
|`├─ translations/`         | Translation files                                                 |
|`├─ config.json`           | Master configuration file                                         |
|`├─ index.html`            | Entry point                                                       |
|`├─ package.json`          | NodeJS configuration file                                         |
|`├─ styleConfig.json`      | Configuration of some application-wide styles                     |
|`├─ themesConfig.json`     | Themes configuration                                              |
|`├─ themes.json`           | Full theme configuration, autogenerated from `themesConfig.json`. |
|`└─ webpack.config.js`     | Webpack dev server and application bundler configuration          |

These files are described in detail in the following sections, in order of importance.

### Application configuration: the `config.json` and `js/appConfig.js` files

The central aspect to keep in mind is that a QWC2 application consists of a collection of plugins, which can be individually enabled and configured, in particular separately for desktop and mobile mode. Together, the `config.json` and `js/appConfig.js` files control these plugins, along with a number of other global settings.

The `js/appConfig.js` file controls the following:
- Which projections are known to the application. Every projection used (as map or data projection) needs to be registered by adding a corresponding

      Proj4js.defs(<EPSG Code>, <proj definition>);

  line. By default, only `EPSG:4326`, `EPSG:4296` and `EPSG:3857` are registered. In addition, you can associate a human readable label to a projection by extending the `CoordinatesUtils.setCrsLabels` call. These labels will be displayed for instance in the mouse coordinate switcher. Alternatively, additional projections can also be registered in `config.json`, see below.
- Which plugins are built into the application. Plugins left out here will be completely omitted when compiling the application bundle, and will hence also reduce the size of the bundle.
- Which locales are supported.

The `config.json` file is the master configuration of the QWC2 application. It contains the following settings:
- URLs of external services used to enhance the application. Sample services are available at [https://github.com/sourcepole/qwc2-server](https://github.com/sourcepole/qwc2-server). The following services can be configured:

| Setting              | Description |
|----------------------|-------------|
|`proxyServiceUrl`     | Ensures replies from cross-origin requests contain the CORS header. See [Cross-Origin requests](#cross-origin-requests).|
|`permalinkServiceUrl` | Generates and resolves compact permalinks for the Share plugin. If omitted, the full URL will be used. |
|`elevationServiceUrl` | Returns elevation values, used to generate a height profile when measuring lines and display elevation information in the map right-click information bubble. If omitted, the respective information will not be displayed in the client.|
|`mapInfoService`      | Returns additional information to be displayed in the map right-click information bubble. If omitted, no additional information will be displayed.|
|`featureReportService`| Returns a custom document associated to a feature. See [`themesConfig.json`](#themesConfig-json).|

- Global settings:

| Setting                             | Description |
|-------------------------------------|-------------|
|`translationsPath`                   | Path from the webserver root to the `translations` folder.                          |
|`assetsPath`                         | Path from the webserver root to the `assets` folder.                                |
|`urlPositionFormat`                  | How to encode the current map extent in the URL, either `centerAndZoom` or `extent`. See [URL parameters](#url-parameters) for details. |
|`urlPositionCrs`                     | The CRS used to encode the current map extent coordinates in the URL.               |
|`preserveExtentOnThemeSwitch`        | Whether to preserve the current map extent when switching theme, if possible.       |
|`preserveBackgroundOnThemeSwitch`    | Whether to preserve the current background layer when switching theme, if possible. |
|`preserveNonThemeLayersOnThemeSwitch`| Whether to preserve non-theme layers when switching theme.                          |
|`allowReorderingLayers`              | Whether to allow re-ordering layers in the layer tree.                              |
|`allowRemovingThemeLayers`           | Whether to allow removing any theme layers from the layer tree.                     |
|`defaultFeatureStyle`                | The default style to use for selection geometries and other unstyled features.      |
|`projections`                        | A list of additional projections to register, in the format `{"code": "<code>", "proj": "<proj4def>", "label": "<label>"}`. |

- The plugin configuration, separately for desktop and for mobile mode. Refer to the [sample `config.json`](https://github.com/qgis/qwc2-demo-app/blob/master/config.json) for a list of available configuration options. You can omit a plugin entry to disable it in desktop and/or mobile mode. To completely remove a plugin from the compiled application, remove the corresponding entry in `js/appConfig.js`.

### Theme configuration: QGIS projects and the `themesConfig.json` file

A theme corresponds to a QGIS project, published as WMS and served by QGIS Server.

#### Creating and publishing a QGIS project

The first step is to prepare a QGIS project. Besides the common tasks of adding and grouping layers, the following table gives an overview of settings which influence how the theme is displayed by QWC2:

| What                 | Where                                     | Description                                      |
|----------------------|-------------------------------------------|--------------------------------------------------|
| Service capabilities | Project Properties &rarr; QGIS Server &rarr; Service capabilities | **Must** be checked for QGIS Server to publish the project. |
| Title, keywords      | Project Properties &rarr; QGIS Server &rarr; Service capabilities | Theme title, displayed in the Theme Switcher, and keywords, useful for filtering. |
| Queryable layers     | Project Properties &rarr; Identify Layers | Mark layers as identifyable by the client.       |
| FeatureInfo geometry | Project Properties &rarr; QGIS Server &rarr; WMS Capabilities &rarr; Add geometry to feature response | Return feature geometries with the GetFeatureInfo request. Allows the client to highlight the selected features. |
| Layer Display Field  | Vector Layer Properties &rarr; Display    | The field used in the identify results. |
| Layer Map Tip        | Vector Layer Properties &rarr; Display    | The contents of the Map Tip shown when hovering over layers in the client, if displaying Map Tips is enabled in the Layer Tree. |
| Layer Metadata       | Layer Properties &rarr; Metadata          | Shown in the client Layer Info dialog, invokable from the Layer Tree. |
| Scale range          | Layer Properties &rarr; Rendering &rarr; Scale dependent visibility | The scale range within which a layer is visible, useful to improve rendering performance. |
| Initial visibility   | Layers Panel                              | Initial visibility of layers and groups.         |
| Rendering order      | Layer Order Panel or Layers Panel         | Rendering order of the layers. If layer re-ordering is enabled in `config.json`, the order from the Layer Order Panel is ignored. |
| Print layouts        | Layout manager                            | The print layouts offered in the Print plugin.   |
| Print layout labels  | Layout manager                            | Print layout labels with an ID will be exposed in the Print plugin. |

#### <a name="themesConfig-json"></a>Configuring the themes in `themesConfig.json`

The second step is to configure the themes which are available to QWC2 in the `themesConfig.json` file, which contains a list of themes, optionally organized in groups, as well as a list of background layers:

    {
      "themes": {
            "items": [
          { <ThemeDefinition> },
          ...
        ], "groups": [
          {
            "title": <Group title>,
            "items": [{ <ThemeDefinition> }, ...],
            "groups": [ { <Group> }, ...]
           },
           ...
        ]
      },
      "backgroundLayers": [
        { <BackgroundLayerDefinition> },
        ...
      ],
      "defaultScales": [<Scale denominators>],
      "defaultPrintScales" [<Scale denominators>],
      "defaultPrintResolutions": [<DPIs>],
      "defaultPrintGrid": [<Print grid, see below>]
    }

Refer to the [sample `themesConfig.json`](https://github.com/qgis/qwc2-demo-app/blob/master/themesConfig.json) for a complete example.

The format of the theme definitions is as follows:
<!-- Important: Use U+00A0 non-breaking spaces ( ) in code blocks -->
| Entry                                        | Description                                                                       |
|----------------------------------------------|-----------------------------------------------------------------------------------|
| `"url": "<WMS URL>",`                        | The address of desired WMS served by QGIS Server.                                 |
| `"title": "<Custom title>"`,                 | Optional, override WMS title.                                                     |
| `"thumbnail": "<Filename>"`,                 | Optional, image file in `assets/img/mapthumbs`. If omitted, autogenerated via WMS GetMap. |
| `"attribution": "<Attribution>",`            | Optional, attribution which will be shown in the bottom right corner of the map.  |
| `"attributionUrl": "<URL>",`                 | Optional, link associated to the attribution                                      |
| `"default": true,`                           | Whether to use this theme as initial theme                                        |
| `"scales": [<Scale denominators>],`          | List of denominators of allowed map scales. If omitted, defaults to `defaultScales`. |
| `"printScales": [<Scale denominators>],`     | List of denominators of allowed print scales. If omitted, defaults to `defaultPrintScales`. |
| `"printResolutions": [<DPIs>],`              | List of available print resolutions. If omitted, defaults to `defaultPrintResolutions`. |
| `"printGrid": [`                             | List of grid scale-dependent grid intervals to use when printing. If omitted, defaults to `defaultPrintGrid`. |
| `  {"s": <Scale1>, x: <Interval1>, y: <Interval1>},` | Keep this list sorted in descending order by scale denominator.           |
| `  {"s": <Scale2>, x: <Interval2>, y: <Interval2>}`  | In this example, `{x: <Interval2>, y: <Interval2>}` will be used for `<Scale1> > Scale >= <Scale2>`.  |
| `],`                                          |                                                                                  |
| `"printLabelForSearchResult": "<ID>",`        | Optional, an ID of a print layout label to which the current search result text (if any) will be written to when printing. |
| `"printLabelConfig": {`                       | Optional, configuration of the text input fields for print layout labels.        |
| `  "<LabelId>": {"rows": <n>, "maxLength": <n>},` | Height of the input field in rows and maximum number of allowed characters. |
| `},`                                          |                                                                                  |
| `"mapCrs: "<EPSG code>",`                     | Optional, map projection, defaults to `EPSG:3857`.                               |
| `"extent": [<xmin>, <ymin>, <xmax>, <ymax>],` | Optional, override theme extent. In `mapCrs`.                                    |
| `"tiled": <boolean>,`                         | Optional, use tiled WMS, defaults to `false`.                                    |
| `"format": "<mimetype>",`                     | Optional, the format to use for WMS GetMap. Defaults to `image/png`.             |
| `"backgroundLayers": [{,`                     | Optional, list of available background layers.                                   |
| `  "name": "<Background layer name>",`        | Name of matching `BackgroundLayerDefinition`, see below.                         |
| `  "printLayer": "<QGis layer name>",`        | Optional, name of layer to use as matching background layer when printing. If omitted, no background is printed. |
| `  "visibility": <boolean>`                   | Optional, initial visibility of the layer when theme is loaded.                  |
| `}],`                                         |                                                                                  |
| `"searchProviders": ["<ProviderId>"],`        | Optional, list of search providers IDs. An ID corresponds to the key of the exported `SearchProviders` object in `js/SearchProviders.js`. |
| `"featureReport": {`                          | Optional, available feature report templates.                                    |
| `  "<LayerId>": "<TemplateID>"  `             | WMS sublayer ID and associated template ID to pass to the `featureReportService`.|
| `},`                                          |                                                                                  |
| `"additionalMouseCrs": ["<EPSG code>"],`      | Optional, list of additional projections for displaying the mouse position. WGS84 and `mapCrs` are available by default. Additional projections definitions must be added to `js/appConfig.js` or `config.json`.         |
| `"watermark": {`                              | Optional, configuration of watermark to add to raster export images.             |
| `  "text": "<text>",`                         | Arbitrary text.                                                                  |
| `  "texpadding": <number>,`                   | Optional, padding between text and frame, in points.                             |
| `  "fontsize": <number>,`                     | Optional, font size.                                                             |
| `  "fontfamily": "<Font family>",`            | Optional, font family.                                                           |
| `  "fontcolor": "#RRGGBB",`                   | Optional, font color.                                                            |
| `  "backgroundcolor": "#RRGGBB",`             | Optional, frame background color.                                                |
| `  "framecolor": "#RRGGBB",`                  | Optional, frame color.                                                           |
| `  "framewidth": <number>,`                   | Optional, frame width.                                                           |
| `},`                                          |                                                                                  |
| `"collapseLayerGroupsBelowLevel": <level>,`   | Optional, layer tree level below which to initially collapse groups. By default the tree is completely expanded. |
| `"skipEmptyFeatureAttributes": <boolean>,`    | Optional, whether to skip empty attributes in the identify results. Default is `false`. |
| `"allowReorderingLayers": <boolean>,`         | Optional, whether to allow reordering layers for this theme. Overrides the setting in `config.json`. |
| `"extraLegendParameters": "<&KEY=VALUE>",`    | Optional, additional query parameters to append to WMS GetLegendGraphic.         |
| `"editConfig": "<editConfig.json>"`           | Optional, path to a filename containing the editing configuration for the theme, see [EditingInterface.js](#editing-interface). |

The format of the background layer definitions is as follows:

| Entry                        | Description                                                                       |
|------------------------------|-----------------------------------------------------------------------------------|
| `"name": "<Name>",`          | The name of the background layer, used in the theme definitions.                  |
| `"title": "<Title>",       ` | The title of the background layer, as displayed in the background switcher.       |
| `"thumbnail": "<Filename>",` | Optional, image file in `assets/img/mapthumbs`. Defaults to `default.jpg`.        |
| `"type": "<Type>",`          | The background layer type, i.e. `wms` or `wmts`.                                  |
| `<Layer params>`             | Parameters according to the specified layer type. Refer to the [sample `themesConfig.json`](https://github.com/qgis/qwc2-demo-app/blob/master/themesConfig.json) for some examples. |

#### Generating `themes.json`

The final step is to generate `themes.json`, the file which is ultimately read by QWC2. This file combines the input from `themesConfig.json` with the information from the WMS service capabilities and is automatically generated when starting the local development application via `yarn start`. Alternatively, it can be manually generated via

    yarn run themesconfig

or, if working in an environment without node, using the equivalent command

    python3 qwc2/scripts/themesConfig.py


If you want to manage multiple `themesConfig.json` files, you can specify which while should be processed by the theme generation script via the `QWC2_THEMES_CONFIG` environment variable.


### Implementing search providers in `js/SearchProviders.js`

Search providers are typically application specific, and hence need to be implemented in the application specific `js/SearchProviders.js` file. The [sample  `js/SearchProviders.js`](https://github.com/qgis/qwc2-demo-app/blob/master/js/SearchProviders.js) documents how to implement search providers and contains some examples.

An advanced feature is the possibility to define parametrized search providers. This allows for instance to implement just a generic search provider interface in `js/SearchProviders.js` and move the implementation details to the service. To this end, you can also add to the `searchProviders` list in `themesConfig.json` entries of the form

    {key: <Key>, label: <Label>, ...}

Such entries are passed to the method `searchProviderFactory` in `js/SearchProviders.js`, which you can tweak to dynamically create a search provider definition based on the parameters specified in the entry. Refer to the [sample  `js/SearchProviders.js`](https://github.com/qgis/qwc2-demo-app/blob/master/js/SearchProviders.js) for an example.

### <a name="editing-interface"></a>Implementing the editing interface in `js/EditingInterface.js`

The QWC2 Editing plugin allows to add, remove and edit features from the map. For this to work, however, the following steps need to be performed:

- The Editing plugin needs to be enabled in `config.json` and `appConfig.js`.
- The interface to the service responsible for writing the changes to the datasets needs to be implemented in `js/EditingInterface.js`.
- For every theme for which editing is allowed, a matching `editConfig.json` needs to be implemented and specified in the corresponding entry in `themesConfig.json`.

A skeleton for the interface to the editing service is provided in the [sample `js/EditingInterface.js`](https://github.com/qgis/qwc2-demo-app/blob/master/js/EditingInterface.js). The actual implementation is service specific.

The format of the `editConfig.json` is as follows:

| Entry                            | Description                                                   |
|----------------------------------|---------------------------------------------------------------|
| `{`                              |                                                               |
| `  <LayerId>: {`                 | A WMS layer ID. Should be a theme WMS layer name, to ensure the WMS is correctly refreshed. |
| `    "layerName": "<LayerName>",`| The layer name to show in the selection combo box. |
| `    "geomType": "<GeomType>",`  | The geometry type, either `Point`, `LineString` or `Polygon`. |
| `    "fields": [{`               | A list of field definitions, for each exposed attribute.      |
| `      "id": "<FieldID>",`       | The field ID.                                                 |
| `      "name": "<FieldName>",`   | The field name, as displayed in the editing form.             |
| `      "type": "<FieldType>",`   | A field type. Either `bool`, `list` or a regular [HTML input element type](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input). |
| `      "constraints": {`         | Constraints for the input field.                              |
| `        "values": [<Entries>],` | Only if `type` is `list`: an array of arbitrary strings.      |
| `        ...`                    | For regular HTML input types, the ReactJS API name of any applicable [HTML input constraint](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input), i.e. `maxLength` or `readOnly`. |
| `      }`                        |                                                               |
| `    }]`                         |                                                               |
| `  }`                            |                                                               |
| `}`                              |                                                               |

See the [sample `editConfig.json`](https://github.com/qgis/qwc2-demo-app/blob/master/test2056_edit.json) for a full example.

### Managing translations

The translations are managed on two levels:

- At QWC2 components level, in `qwc2/translations`.
- At application level, in `translations`.

A script will take care of merging the component translations into the application translations. This way, when updating the QWC2 submodule, new translations are automatically obtained. This script is automatically invoked on `yarn start`, but can also be manually invoked using

    yarn run tsupdate

Translations are stored inside the respective `translations` folder as regular plain-text JSON files, named `data.<locale>` and can be freely edited with any text editor.

The `tsconfig.json` files stored inside the respective `translations` folder contains the list of languages for which translations should be generated and a list of message IDs to include in the translation. The `tsupdate` script will read this file to automatically create resp. update the translation files.

The typical workflow for managing application translations is:

- Declare all locales which should be supported in `js/appConfig.js`.
- Ensure these locales are listed in `translations/tsconfig.json`.
- Add message IDs of any custom components (which are not part of the common QWC2 components) in `translations/tsconfig.json`.
- Write the translated strings to the respective `data.<locale>` files.
- Test the translation, specifying `locale=<locale>` in the QWC2 application URL.

If translations of the QWC2 components for a desired language are missing, please create resp. update the translations respective files in `qwc2/translations` and contribute them by submitting a pull request to the [upstream qwc2 repository](https://github.com/qgis/qwc2).

### Customizing the QWC2 appearance

The following options are available for customizing the appearance of the QWC2 application while preserving compatibility with the core QWC2 components:

- Modifying the logo in `assets/img/`.
- Modifying the application icons in `icons`.
- Tweaking the colors in `styleConfig.js`.
- Adding style declarations to the master CSS stylesheet `assets/css/qwc2.css`. This however is potentially fragile and should only be done as a last resort.
- Changing the browser page title in `index.html`, and potentially adding a favicon.
- Modifying the legend print template in `assets/templates/legendprint.html`. The only requirement for this template is that is must contain a `<div id="legendcontainer"></div>` element.
- Modifying the custom Help component `js/Help.jsx`.


## Server-side configuration
### <a name="cross-origin-requests"></a>Cross-Origin requests
All modern browsers will block a page from requesting resources from another origin (except for images, stylesheets, scripts, iframes and videos), unless the response from the remote origin contains a matching `Access-Control-Allow-Origin` header. An origin is defined as `<scheme>://<hostname>:<port>`.

For each service QWC2 interacts with, in particular the QGIS Server, one has to ensure that this interaction isn't blocked by the browser. The following options exist:

- Ensure that the service runs on the same origin as the web server which serves the QWC2 application.
- Ensure that the service sends a `Access-Control-Allow-Origin` header with matching origin with each response.
- Use a proxy service which ensures the `Access-Control-Allow-Origin` are added. The URL to this service needs to be specified as `proxyServiceUrl` in the `config.json` file. A sample implementation of such a proxy service is available [in the qwc2-server repository](https://github.com/sourcepole/qwc2-server).

### Filenames of print and raster and DXF export
The QGIS server response for the print, raster and DXF export requests does by default not contain any `Content-Disposition` header, meaning that browsers will attempt to guess a filename, which typically is the last part of the URL, without any extension.

To ensure browser use a proper filename, the following options exist:

- Configure the web server running QGIS Server to add a suitable `Content-Disposition` header to the response. In the case of Apache, the rule for the print output might look as follows:

      SetEnvIf Request_URI "^/wms.*/(.+)$" project_name=$1
      Header always setifempty Content-Disposition "attachment; filename=%{project_name}.pdf" "expr=%{CONTENT_TYPE} = 'application/pdf'"

  This rule will use the last part of the URL as basename and add the `.pdf` extension, and will also ensure that the content-type is set to `application/pdf`. Note that this example uses the `setenvif` and `headers` apache modules.
- Use a proxy service which adds a `Content-Disposition` header if necessary. QWC2 helps in this regard by automatically adding a `filename=<filename>` query parameter if `proxyServiceUrl` is set in `config.json` for requests which are expected to return a downloadable file. See the sample proxy service [in the qwc2-server repository](https://github.com/sourcepole/qwc2-server) for details.

## <a name="url-parameters"></a>URL parameters
The following parameters can appear in the URL of the QWC2 application:

- `t`: The active theme
- `l`: The visible layers in the map
- `bl`: The visible background layer
- `st`: The search text
- `e`: The visible extent
- `c`: The center of the visible extent
- `s`: The current scale
- `crs`: The CRS of extent/center coordinates

The `urlPositionFormat` parameter in `config.json` determines whether the extent or the center and scale appears in the URL.

The `urlPositionCrs` parameter in `config.json` determines the projection to use for the extent resp. center coordinates in the URL. By default the map projection of the current theme is used. If `urlPositionCrs` is equal to the map projection, the `crs` parameter is omitted in the URL.

## Keeping the QWC2 application up to date
As mentioned in the [quick start](#quick-start) chapter, QWC2 is split into a common components repository and an application specific repository. The goal of this approach is to cleanly separate user-specific configuration and components which common components which serve as a basis for all QWC2 applications, and to make it as easy as possible to rebase the application onto the latest common QWC2 components.

The recommended workflow is to keep the `qwc2` folder a submodule referencing the [upstream qwc2 repository](https://github.com/qgis/qwc2). To update it, just checkout the desired tag, i.e.:

    cd qwc2
    git fetch
    git checkout <tag>

or update to the latest `master`, i.e.:

    cd qwc2
    git checkout master
    git pull

The [ChangeLog in the demo application repository](https://github.com/qgis/qwc2-demo-app/blob/master/ChangeLog.md) documents major changes, and in particular all incompatible changes between releases which require changes to the application specific code and/or configuration.

## Developing
QWC2 is written in JavaScript using in particular the ReactJS, Redux and OpenLayers libraries. The following links point to some useful resources to learn the basics:

**ECMAScript 2015**
- https://babeljs.io/docs/learn-es2015/

**ReactJS**

- https://facebook.github.io/react/docs/thinking-in-react.html
- https://medium.com/@diamondgfx/learning-react-with-create-react-app-part-1-a12e1833fdc
- https://medium.com/@diamondgfx/learning-react-with-create-react-app-part-2-3ad99f38b48d
- https://medium.com/@diamondgfx/learning-react-with-create-react-app-part-3-322447d14192

**Redux**

- http://redux.js.org/
- https://egghead.io/courses/getting-started-with-redux
- https://egghead.io/courses/building-react-applications-with-idiomatic-redux

When developing, it is useful to add `debug=true` to the URL query parameters, which will enable logging of all application state changes to the browser console.


